SystemOrganization addCategory: #DaliotsMappers!
SystemOrganization addCategory: #'DaliotsMappers-test'!

!Interval methodsFor: '*DaliotsMappers' stamp: 'hjo 7/29/2008 16:40'!
interPointOfRatio: aNumber

	^(self stop - self start) * aNumber + self start
! !

!Interval methodsFor: '*DaliotsMappers' stamp: 'hjo 7/29/2008 16:40'!
interRatioOf: aNumber

	^(aNumber - self start)/(self stop - self start)
! !

TestCase subclass: #DaliotsMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers-test'!

!DaliotsMapperTest methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/21/2012 11:55'!
testIntervalMapper
	| m |
	m := RangeMapper from: (Range from: -1 to: 3) to: (Range from: 10 to: 90).
	self assert: (m map: 0) equals: 30.
	self assert: (m unmap: 30) equals: 0! !

!DaliotsMapperTest methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/20/2012 21:28'!
testLinearMapper
	| m |
	m := LinearMapper mapsDomainFrom: -1 to: 3 toRangeFrom: 10 to: 90.
	self assert: (m map: 0) equals: 30.
	self assert: (m unmap: 30) equals: 0! !

!DaliotsMapperTest methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/21/2012 02:07'!
testMultiRangeMapper
	| m |
	m := MultiRangeMapper from: {1. 2. 3. 4} to: {10. 20. 40. 50}.
	self assert: (m map: 0) equals: 0.
	self assert: (m map: 1) equals: 10.
	self assert: (m map: 1.5) equals: 15.
	self assert: (m map: 2) equals: 20.
	self assert: (m map: 2.5) equals: 30.
	self assert: (m map: 3) equals: 40.
	self assert: (m map: 3.5) equals: 45.
	self assert: (m map: 4) equals: 50.
	
	self assert: (m unmap: 10) equals: 1.
	self assert: (m unmap: 15) equals: 1.5.
	self assert: (m unmap: 20) equals: 2.
	self assert: (m unmap: 30) equals: 2.5.
	self assert: (m unmap: 40) equals: 3.
	self assert: (m unmap: 45) equals: 3.5.
	self assert: (m unmap: 50) equals: 4.! !

!DaliotsMapperTest methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/20/2012 21:42'!
testRectangleMapper
	| m |
	m := RectangleMapper from: (0@0 corner: 1@1) to: (100@ 200 corner: 300@400).
	self assert: (m map: 0@0) equals: (100@ 200).
	self assert: (m map: 1@0) equals: (300@ 200).
	self assert: (m map: 0@1) equals: (100@ 400).
	self assert: (m map: 1@1) equals: (300@ 400).
	! !

!DaliotsMapperTest methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/20/2012 23:08'!
testRectangleMapperRectangle
	| m |
	m := RectangleMapper from: (0@0 corner: 1@1) to: (100@ 200 corner: 300@400).
	self assert: (m mapRectangle: (0@0 corner: 0.5 @ 0.5)) equals: (100@ 200 corner: 200@300).
	self assert: (m unmapRectangle: (100@ 200 corner: 200@300)) equals: (0@0 corner: 0.5 @ 0.5).! !

!Rectangle methodsFor: '*DaliotsMappers' stamp: 'hjo 7/29/2008 16:39'!
interPointOfRatio: aPoint

	^self extent * aPoint + self origin

! !

!Rectangle methodsFor: '*DaliotsMappers' stamp: 'hjo 7/29/2008 16:39'!
interRatioOf: aPoint

	|extent displacement|
	extent := self extent.
	displacement := aPoint - origin.
	^(displacement x / extent x)@(displacement y / extent y)! !

Object subclass: #LinearMapFunction
	instanceVariableNames: 'ratio bias'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!LinearMapFunction methodsFor: 'accessing' stamp: 'HwaJong 12/20/2012 22:19'!
bias
	^bias! !

!LinearMapFunction methodsFor: 'accessing' stamp: 'HwaJong 12/20/2012 22:19'!
bias: aNumber  
	bias := aNumber! !

!LinearMapFunction methodsFor: 'accessing' stamp: 'HwaJong 12/20/2012 21:46'!
ratio
	^ ratio! !

!LinearMapFunction methodsFor: 'accessing' stamp: 'HwaJong 12/20/2012 22:19'!
ratio: aNumber
	ratio := aNumber! !

!LinearMapFunction methodsFor: 'evaluating' stamp: 'HwaJong 12/20/2012 22:19'!
value: aNumber 
	^aNumber * ratio + bias ! !

Object subclass: #LinearMapper
	instanceVariableNames: 'domainFrom rangeFrom mapRatio unmapRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!LinearMapper classSide methodsFor: 'as yet unclassified' stamp: 'hjo 6/10/2008 15:13'!
mapsDomainFrom: aSmallInteger to: aSmallInteger2 toRangeFrom: aSmallInteger3 to: aSmallInteger4 

	^self new
		domainFrom: aSmallInteger;
		rangeFrom: aSmallInteger3;
		mapRatio: (aSmallInteger4-aSmallInteger3) / (aSmallInteger2-aSmallInteger);
		setUnmapRatioFromMapRatio! !

!LinearMapper methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/20/2012 17:40'!
domainFrom
	^domainFrom! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/10/2008 15:14'!
domainFrom: aSmallInteger 
	
	domainFrom := aSmallInteger 
! !

!LinearMapper methodsFor: 'as yet unclassified' stamp: 'hjo 6/12/2008 09:08'!
map: aFloat 
	
	^(aFloat - self domainFrom) * self mapRatio + self rangeFrom! !

!LinearMapper methodsFor: 'as yet unclassified' stamp: 'hjo 6/30/2008 14:56'!
mapAll: anArray

	^anArray collect: [ : each | self map: each ]! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/12/2008 09:06'!
mapRatio
	^mapRatio! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/12/2008 09:05'!
mapRatio: aSmallInteger 

	mapRatio := aSmallInteger 
! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/12/2008 09:08'!
rangeFrom

	^rangeFrom! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/12/2008 09:05'!
rangeFrom: aSmallInteger 

	rangeFrom := aSmallInteger ! !

!LinearMapper methodsFor: 'as yet unclassified' stamp: 'hjo 6/12/2008 09:06'!
setUnmapRatioFromMapRatio

	self unmapRatio: self mapRatio reciprocal! !

!LinearMapper methodsFor: 'as yet unclassified' stamp: 'hjo 6/12/2008 09:11'!
unmap: aSmallInteger 

	^(aSmallInteger - self rangeFrom) * self unmapRatio + self domainFrom! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/12/2008 09:11'!
unmapRatio

	^unmapRatio! !

!LinearMapper methodsFor: 'accessing' stamp: 'hjo 6/12/2008 09:07'!
unmapRatio: aFraction 

	unmapRatio := aFraction ! !

Object subclass: #MapperAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!MapperAbstract classSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/29/2008 16:30'!
from: object1 to: object2

	^self new from: object1 to: object2

! !

!MapperAbstract classSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/29/2008 16:31'!
icon

	"This method is for Dolphin Smalltalk.
	^Icon fromFile: 'icon\mapper.ico'"! !

!MapperAbstract methodsFor: 'initialize-release' stamp: 'hjo 7/29/2008 16:29'!
from: domain to: range

	self subclassResponsibility

! !

!MapperAbstract methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 16:29'!
map: key

	self subclassResponsibility
! !

!MapperAbstract methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 16:30'!
unmap: value

	self subclassResponsibility
! !

MapperAbstract subclass: #MultiRangeMapper
	instanceVariableNames: 'intervalMappers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!MultiRangeMapper classSide methodsFor: 'as yet unclassified' stamp: 'HwaJong 12/21/2012 02:30'!
newWithAssociations: arrayOfAssociations
	^self new initializeWithAssociations: arrayOfAssociations! !

!MultiRangeMapper methodsFor: 'initialize-release' stamp: 'HwaJong 12/21/2012 11:56'!
from: sortedNumbers to: sortedNumbers2

	sortedNumbers size = sortedNumbers2 size ifFalse: [self error: 'size of two array(sorted) should be same.'].
	intervalMappers := (2 to: sortedNumbers size) collect: [:k|
		RangeMapper 
			from: (Range from: (sortedNumbers at: k-1) to: (sortedNumbers at: k)) 
			to: (Range from: (sortedNumbers2 at: k-1) to: (sortedNumbers2 at: k)) ].
	intervalMappers := intervalMappers reversed.! !

!MultiRangeMapper methodsFor: 'initialize-release' stamp: 'HwaJong 12/21/2012 02:29'!
initializeWithAssociations: arrayOfAssociations
	self from: (arrayOfAssociations collect: #key) to: (arrayOfAssociations collect: #value)! !

!MultiRangeMapper methodsFor: 'evaluating' stamp: 'HwaJong 12/21/2012 02:08'!
map: aNumber
	^(intervalMappers detect: [:each| each domainStart <= aNumber ] ifNone: [intervalMappers last]) map: aNumber.! !

!MultiRangeMapper methodsFor: 'evaluating' stamp: 'HwaJong 12/20/2012 22:53'!
unmap: aNumber
	^(intervalMappers detect: [:each| each rangeStart <= aNumber]) unmap: aNumber! !

MapperAbstract subclass: #QuadrangleMapper
	instanceVariableNames: 'domainQuadrangle rangeQuadrangle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!QuadrangleMapper methodsFor: 'initialize-release' stamp: 'hjo 7/29/2008 16:31'!
from: quadrangle1 to: quadrangle2

	domainQuadrangle := quadrangle1.
	rangeQuadrangle := quadrangle2
! !

!QuadrangleMapper methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 16:32'!
map: aPoint

	^rangeQuadrangle pointOfDivisionRatio: (domainQuadrangle divisionRatioOf: aPoint)

! !

!QuadrangleMapper methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 16:32'!
unmap: aPoint

	^domainQuadrangle pointOfDivisionRatio: (rangeQuadrangle divisionRatioOf: aPoint)

! !

MapperAbstract subclass: #RangeMapper
	instanceVariableNames: 'foreLinFun backLinFun'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!RangeMapper methodsFor: 'accessing' stamp: 'HwaJong 12/20/2012 21:44'!
backLinFun: anObject
	backLinFun := anObject! !

!RangeMapper methodsFor: 'self evaluating' stamp: 'HwaJong 12/20/2012 22:50'!
domainStart
	^backLinFun bias! !

!RangeMapper methodsFor: 'accessing' stamp: 'HwaJong 12/20/2012 21:44'!
foreLinFun: anObject
	foreLinFun := anObject! !

!RangeMapper methodsFor: 'initialize-release' stamp: 'HwaJong 12/21/2012 11:54'!
from: domainRange to: rangeRange
	| rangeDiff domainDiff |
	rangeDiff := rangeRange span.
	rangeDiff isZero ifTrue: [ self error: 'rangeDiff=0 ', domainRange printString , ' : ' , rangeRange printString ].
	domainDiff := domainRange span.	
	domainDiff isZero ifTrue: [ self error: 'domainDiff=0 ', domainRange printString , ' : ' , rangeRange printString ].
	foreLinFun := LinearMapFunction new
		ratio: rangeDiff / domainDiff;
		bias: rangeRange start.
	backLinFun := LinearMapFunction new
		ratio: domainDiff / rangeDiff;
	 	bias: domainRange start! !

!RangeMapper methodsFor: 'initialize-release' stamp: 'HwaJong 12/20/2012 22:22'!
initialize
	super initialize.
	foreLinFun := LinearMapFunction new.
	backLinFun := LinearMapFunction new.! !

!RangeMapper methodsFor: 'evaluating' stamp: 'HwaJong 12/20/2012 22:17'!
map: aNumber
	^foreLinFun value: (aNumber - backLinFun bias)! !

!RangeMapper methodsFor: 'evaluating' stamp: 'HwaJong 12/20/2012 21:53'!
mapSize: aNumber
	^ aNumber * foreLinFun ratio! !

!RangeMapper methodsFor: 'self evaluating' stamp: 'HwaJong 12/20/2012 22:54'!
rangeStart
	^foreLinFun bias! !

!RangeMapper methodsFor: 'converting' stamp: 'HwaJong 12/20/2012 22:03'!
reversed
	^ self class new
		foreLinFun: backLinFun;
		backLinFun: foreLinFun! !

!RangeMapper methodsFor: 'evaluating' stamp: 'HwaJong 12/20/2012 22:18'!
unmap: aNumber
	^backLinFun value: (aNumber - foreLinFun bias)! !

!RangeMapper methodsFor: 'evaluating' stamp: 'HwaJong 12/20/2012 21:51'!
unmapSize: aNumber
	^ aNumber * backLinFun ratio! !

MapperAbstract subclass: #RectangleMapper
	instanceVariableNames: 'xMapper yMapper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DaliotsMappers'!

!RectangleMapper classSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/29/2008 14:50'!
new

	^super new initialize! !

!RectangleMapper methodsFor: 'self evaluating' stamp: 'hjo 7/29/2008 14:48'!
foreRatio

	^self xMapper foreRatio @ self yMapper foreRatio! !

!RectangleMapper methodsFor: 'initialize-release' stamp: 'HwaJong 12/21/2012 11:58'!
from: rect1 to: rect2

	xMapper from: (Range from: rect1 left to: rect1 right)  to: (Range from: rect2 left to: rect2 right).
	yMapper from: (Range from: rect1 top to: rect1 bottom) to: (Range from: rect2 top to: rect2 bottom)

! !

!RectangleMapper methodsFor: 'initialize-release' stamp: 'HwaJong 12/21/2012 11:50'!
initialize

	super initialize.
	xMapper := RangeMapper new.
	yMapper := RangeMapper new.

! !

!RectangleMapper methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 14:49'!
map: aPoint

	^(xMapper map: aPoint x)@(yMapper map: aPoint y)

! !

!RectangleMapper methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 14:49'!
mapRectangle: aRectangle

	^(self map: aRectangle origin) corner: (self map: aRectangle corner)

! !

!RectangleMapper methodsFor: 'converting' stamp: 'hjo 7/29/2008 14:49'!
reversed

	^self class new 
		xMapper: self xMapper reversed;
		yMapper: self xMapper reversed

! !

!RectangleMapper methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 14:49'!
unmap: aPoint

	^(xMapper unmap: aPoint x)@(yMapper unmap: aPoint y)

! !

!RectangleMapper methodsFor: 'evaluating' stamp: 'hjo 7/29/2008 14:49'!
unmapRectangle: aRectangle

	^(self unmap: aRectangle origin) corner: (self unmap: aRectangle corner)

! !

!RectangleMapper methodsFor: 'accessing' stamp: 'hjo 7/29/2008 14:49'!
xMapper

	^xMapper

! !

!RectangleMapper methodsFor: 'accessing' stamp: 'hjo 7/29/2008 14:49'!
xMapper: aIntervalMapper

	xMapper := aIntervalMapper

! !

!RectangleMapper methodsFor: 'accessing' stamp: 'hjo 7/29/2008 14:50'!
yMapper

	^yMapper

! !

!RectangleMapper methodsFor: 'accessing' stamp: 'hjo 7/29/2008 14:50'!
yMapper: aIntervalMapper

	yMapper := aIntervalMapper

! !
